package com.link.core.basic.service;

import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.link.core.basic.dao.mapper.BasicMapper;
import com.link.core.basic.database.DataSource;
import com.link.core.basic.database.DataSourceType;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.ExportColumn;
import com.link.core.modules.export.ExportWriter;
import com.link.core.modules.export.writer.AbstractFileWriter;
import com.link.core.modules.export.writer.CsvWriter;
import java.util.List;

public interface BasicService<T extends BasicModel> {
   String ROW_STATUS_UPDATE = "UPDATE";
   String ROW_STATUS_INSERT = "NEW";
   String ROW_STATUS_DELETE = "DEL";
   String GUIDE_APP = "GUIDE_APP";
   String DMS_APP = "DMS_APP";
   String SFA_APP = "SFA_APP";
   String CRM_APP = "CRM_APP";
   String MP = "MP";
   String H5 = "H5";
   String CRM_PC = "CRM-PC";
   String DMS_PC = "DMS-PC";

   BasicMapper<T> getBasicMapper() throws Exception;

   void insert(T entity) throws Exception;

   void batchInsert(List<T> list) throws Exception;

   void batchInsert(List<T> list, boolean useMyBatisBatch) throws Exception;

   void batchUpsert(List<T> list) throws Exception;

   T preDefaultValue(T entity) throws Exception;

   T postDefaultValue(T entity) throws Exception;

   void upsert(T entity) throws Exception;

   void update(T entity) throws Exception;

   T queryById(T entity) throws Exception;

   T queryById(String entityId) throws Exception;

   void deleteById(T entity) throws Exception;

   void deleteById(String entityId) throws Exception;

   void batchDelete(List<T> list) throws Exception;

   @DataSource(DataSourceType.RANDOM)
   List<T> queryAll(T entity) throws Exception;

   @DataSource(DataSourceType.RANDOM)
   List<T> queryByExamplePage(T entity) throws Exception;

   List<T> queryAllByExamplePage(T entity) throws Exception;

   boolean checkData(T entity) throws Exception;

   void beforQueryAllOrExample(T entity) throws Exception;

   void beforQueryAll(T entity) throws Exception;

   void beforQueryExample(T entity) throws Exception;

   void beforStreamQueryAll(T entity) throws Exception;

   void afterStreamQueryAllMulti(List<T> list) throws Exception;

   void beforUpdate(T entity) throws Exception;

   void afterUpdate(T entity) throws Exception;

   void beforInsert(T entity) throws Exception;

   void afterInsert(T entity) throws Exception;

   void beforDelete(T entity) throws Exception;

   void beforUpsert(T entity) throws Exception;

   void afterUpsert(T entity) throws Exception;

   boolean isInsertFlag(T entity) throws Exception;

   void batchUpdate(List<T> list) throws Exception;

   void afterDelete(T entity) throws Exception;

   void afterQuery(T entity, List<T> list) throws Exception;

   boolean isExistedById(T entity) throws Exception;

   T queryOneByProperty(T entity, String propertyName, String operator) throws Exception;

   void updateSpecifiedFields(T entity, String... fieldNames) throws Exception;

   void streamQueryExport(T entity, ExportWriter exportWriter) throws Exception;

   void exportCsv(T entity, List<ExportColumn> exportColumns, CsvWriter writer) throws Exception;

   void exportExcel(T entity, List<ExportColumn> exportColumns, ExcelWriter excelWriter, WriteSheet sheet, AbstractFileWriter fileWriter) throws Exception;

   void afterStreamQueryAll(T exportData) throws Exception;
}
